#!/bin/bash
set -u

[ $# -ne 0 ] || {
  cat <<USAGE
Usage: validate.sh DIR [DIR...]

Program validates all files in all given directories. It ignores files
with .tmp extension.

Author: Volodymyr M. Lisivka <vlisikva@gmail.com>

Expected file format:
################################
Iteration: 10_Number of patterns: 346_Pattern: 156167618348_156167618348_...
################################

USAGE
  exit 0
}

EXIT_CODE=0

validate_file() {
  local FILE="$1"

  ( # In subshell
    set -eu # Break on first error

    # Read header
    read -r -d '_' NOT_USED ITERATION
    read -r -d '_' NOT_USED NOT_USED NOT_USED NUMBER_OF_PATTERNS
    read -r -d '_' NOT_USED PATTERN
    
    [[ "$NUMBER_OF_PATTERNS" > 0 ]] || {
        echo "ERROR: Number of patterns is wrong in \"$FILE\" file. Expected: a number, actual: \"$NUMBER_OF_PATTERNS\"." >&2
    }

    # Read pattern
    for((I=0; I<NUMBER_OF_PATTERNS; I++))
    do
      read -r -d '_' LINE
      [ "$LINE" == "$PATTERN" ] || {
        echo "ERROR: Pattern is wrong in \"$FILE\" file. Expected: \"$PATTERN\", actual: \"$LINE\"." >&2
        exit 1 # Exit from subshell
      }
    done

    read -r LINE
    [ "$LINE" == "" ] || {
      echo "ERROR: Unexcepted data after end of data block in \"$FILE\" file: \"$LINE\"." >&2
      exit 1 # Exit from subshell
    }
  ) <"$FILE" || {
    EXIT_CODE=1
  }
}

for DIR in "$@"
do
  # Skip non-directories
  [ -d "$DIR" ] || continue

  for FILE in "$DIR"/*
  do
    # Skip non-files
    [ -f "$FILE" ] || continue

    # Skip files with .tmp suffix
    [[ "$FILE" != *.tmp ]] || continue

    # Validate file
    validate_file "$FILE"
  done
done


exit $EXIT_CODE
